/*-----------------------------------------------------------------------
								 \\\|///
							   \\  - -  //
								(  @ @  )
+-----------------------------oOOo-(_)-oOOo-----------------------------+
CONFIDENTIAL IN CONFIDENCE
This confidential and proprietary software may be only used as authorized
by a licensing agreement from CrazyBingo (Thereturnofbingo).
In the event of publication, the following notice is applicable:
Copyright (C) 2013-20xx CrazyBingo Corporation
The entire notice above must be reproduced on all authorized copies.
Author				:		CrazyBingo
Technology blogs 	: 		http://blog.chinaaet.com/crazybingo
Email Address 		: 		thereturnofbingo@gmail.com
Filename			:		SDRAM_VGA_Display_Test.v
Data				:		2012-03-18
Description			:		Sdram_Control_2Port test with VGA Display.
Modification History	:
Data			By			Version			Change Description
=========================================================================
12/03/18		CrazyBingo	1.0				Original
12/03/21		CrazyBingo	1.1				Modification
12/05/09		CrazyBingo	1.2				Modification
12/05/13		CrazyBingo	1.3				Modification
12/06/01		CrazyBingo	1.4				Modification
12/06/04		CrazyBingo	1.5				Modification
13/04/10		CrazyBingo	1.6				Modification
13/09/21		CrazyBingo	1.7				Modification
13/11/07		CrazyBingo	2.0				Modification
13/11/16		CrazyBingo	2.1				Modification
-------------------------------------------------------------------------
|                                     Oooo								|
+------------------------------oooO--(   )-----------------------------+
                              (   )   ) /
                               \ (   (_/
                                \_)
----------------------------------------------------------------------*/ 

`timescale 1ns / 1ns
module SDRAM_VGA_Display_Test
(
	//global clock 50MHz
	input			clk,			//50MHz
	input			rst_n,			//global reset
	
	//sdram control
	output			sdram_clk,		//sdram clock
	output			sdram_cke,		//sdram clock enable
	output			sdram_cs_n,		//sdram chip select
	output			sdram_we_n,		//sdram write enable
	output			sdram_cas_n,	//sdram column address strobe
	output			sdram_ras_n,	//sdram row address strobe
//	output	[3:0]	sdram_dqm,		//sdram data enable
	output	[1:0]	sdram_ba,		//sdram bank address
	output	[11:0]	sdram_addr,		//sdram address
	inout	[23:0]	sdram_data,		//sdram data
	
	//lcd port
	output			lcd_dclk,		//lcd pixel clock			
	output			lcd_hs,			//lcd horizontal sync 
	output			lcd_vs,			//lcd vertical sync
//	output			lcd_sync,		//lcd sync
//	output			lcd_blank,		//lcd blank(L:blank)
	output	[7:0]	lcd_red,		//lcd red data
	output	[7:0]	lcd_green,		//lcd green data
	output	[7:0]	lcd_blue,		//lcd blue data
	output			lcd_pwm,		//lcd bright pwm control
	output			lcd_reset		//lcd disp reset
);
assign	lcd_pwm = 1'b0;
assign	lcd_reset = sys_rst_n;


//---------------------------------------------
//system global clock control
wire	clk_vga;		//vga clock
wire	clk_ref;		//sdram ctrl clock
wire	clk_refout;		//sdram clock output
wire	sys_rst_n;		//global reset
system_ctrl_pll	u_system_ctrl_pll
(
	.clk				(clk),			//global clock
	.rst_n				(rst_n),		//external reset
	
	.sys_rst_n			(sys_rst_n),	//global reset
	.clk_c0				(clk_ref),		//100MHz 
	.clk_c1				(clk_refout),	//100MHz -90deg
	.clk_c2				(clk_vga)		//40MHz
);


//-------------------------------------------
//Generate LCD Test picture
wire	[23:0]	sys_data;
wire			sys_we;
VGAData_Simulate_24Bit	u_VGAData_Simulate_24Bit
(
	.clk				(clk_ref),		
	.rst_n				(sys_rst_n),     
	
	.sys_vaild			(sdram_init_done),
	.DIVIDE_PARAM		(8'd128),
	.sys_data			(sys_data),	
	.sys_we				(sys_we)
);  


//-------------------------------------
//Sdram_Control_2Port module 	
//sdram write port
wire			clk_write	=	clk_ref;	//Change with input signal											
wire	[23:0]	sys_data_in = 	sys_data;
wire			sys_we_in 	= 	sys_we;
//sdram read port
wire			clk_read	=	clk_vga;	//Change with vga timing	
wire	[23:0]	sys_data_out;
wire			sys_rd_out;
wire			sdram_init_done;			//sdram init done
Sdram_Control_2Port	u_Sdram_Control_2Port
(
	//	HOST Side
	.REF_CLK			(clk_ref),			//sdram module clock
	.OUT_CLK			(clk_refout),		//sdram clock input
	.RESET_N			(sys_rst_n),		//sdram module reset

	//	SDRAM Side
	.SDR_CLK			(sdram_clk),		//sdram clock
	.CKE				(sdram_cke),		//sdram clock enable
	.CS_N				(sdram_cs_n),		//sdram chip select
	.WE_N				(sdram_we_n),		//sdram write enable
	.CAS_N				(sdram_cas_n),		//sdram column address strobe
	.RAS_N				(sdram_ras_n),		//sdram row address strobe
	.DQM				(),//(sdram_dqm),		//sdram data output enable
	.BA					(sdram_ba),			//sdram bank address
	.SA					(sdram_addr),		//sdram address
	.DQ					(sdram_data),		//sdram data

	//	FIFO Write Side
	.WR_CLK				(clk_write),		//write fifo clock
	.WR_LOAD			(1'b0),				//write register load & fifo clear
	.WR_DATA			(sys_data_in),		//write data input
	.WR					(sys_we_in),			//write data request
	.WR_MIN_ADDR		(22'd0),			//write start address
	.WR_MAX_ADDR		(22'd800 * 22'd480),	//write max address
	.WR_LENGTH			(9'd256),			//write burst length

	//	FIFO Read Side
	.RD_CLK				(clk_read),			//read fifo clock
	.RD_LOAD			(1'b0),				//read register load & fifo clear
	.RD_DATA			(sys_data_out),		//read data output
	.RD					(sys_rd_out),			//read request
	.RD_MIN_ADDR		(22'd0),            //read start address
	.RD_MAX_ADDR		(22'd800 * 22'd480),	//read max address
	.RD_LENGTH			(9'd256),			//read length
	
	//User interface add by CrazyBingo
	.Sdram_Init_Done	(sdram_init_done),	//SDRAM init done signal
	.Sdram_Read_Valid	(1'b1),				//Enable to read
	.Sdram_PingPong_EN	(1'b0)				//SDRAM PING-PONG operation enable
);


//-------------------------------------
//LCD driver timing
lcd_driver u_lcd_driver
(
	//global clock
	.clk			(clk_vga),		
	.rst_n			(sys_rst_n), 
	 
	 //lcd interface
	.lcd_dclk		(lcd_dclk),
	.lcd_blank		(lcd_blank),
	.lcd_sync		(),//(lcd_sync),		    	
	.lcd_hs			(lcd_hs),		
	.lcd_vs			(lcd_vs),
	.lcd_en			(),		
	.lcd_rgb		({lcd_red, lcd_green, lcd_blue}),

	
	//user interface
	.lcd_request	(sys_rd_out),
	.lcd_data		(sys_data_out),	
	.lcd_xpos		(),	
	.lcd_ypos		()
);

endmodule
